@@ -13,6 +13,7 @@ from __future__ import print_function
 
 import logging
 import threading
+import sys
 import os
 from os.path import join, dirname, abspath
 from argparse import ArgumentTypeError
@@ -23,9 +24,10 @@ except ImportError:
     # Python 2
     from ConfigParser import RawConfigParser  # pylint: disable=import-error
 
-from vunit.ostools import Process, write_file, read_file, file_exists
+from vunit.ostools import Process, write_file, file_exists
 from vunit.simulator_interface import SimulatorInterface
 from vunit.exceptions import CompileError
+from vunit.test_runner import HASH_TO_TEST_NAME
 
 LOGGER = logging.getLogger(__name__)
 
@@ -98,7 +100,6 @@ class ModelSimInterface(SimulatorInterface):  # pylint: disable=too-many-instanc
                                   constraints=[has_modelsim_ini])
 
     def __init__(self, prefix, modelsim_ini="modelsim.ini", persistent=False, gui_mode=None, coverage=None):
-        self._vhdl_standard = None
         self._modelsim_ini = abspath(modelsim_ini)
 
         # Workaround for Microsemi 10.3a which does not
@@ -151,14 +152,20 @@ class ModelSimInterface(SimulatorInterface):  # pylint: disable=too-many-instanc
         """
         if file_exists(self._modelsim_ini):
             return
-        write_file(self._modelsim_ini, read_file(join(self._prefix, "..", "modelsim.ini")))
 
-    def setup_library_mapping(self, project, vhdl_standard):
+        parent = dirname(self._modelsim_ini)
+        if not file_exists(parent):
+            os.makedirs(parent)
+
+        with open(join(self._prefix, "..", "modelsim.ini"), 'rb') as fread:
+            with open(self._modelsim_ini, 'wb') as fwrite:
+                fwrite.write(fread.read())
+
+    def setup_library_mapping(self, project):
         """
         Setup library mapping
         """
         mapped_libraries = self._get_mapped_libraries()
-        self._vhdl_standard = vhdl_standard
 
         for library in project.get_libraries():
             self._libraries[library.name] = library
@@ -169,16 +176,14 @@ class ModelSimInterface(SimulatorInterface):  # pylint: disable=too-many-instanc
         Returns the command to compile a single source file
         """
         if source_file.file_type == 'vhdl':
-            return self.compile_vhdl_file_command(source_file.name,
-                                                  source_file.library.name,
-                                                  source_file.compile_options)
+            return self.compile_vhdl_file_command(source_file)
         elif source_file.file_type == 'verilog':
             return self.compile_verilog_file_command(source_file)
 
         LOGGER.error("Unknown file type: %s", source_file.file_type)
         raise CompileError
 
-    def compile_vhdl_file_command(self, source_file_name, library_name, compile_options):
+    def compile_vhdl_file_command(self, source_file):
         """
         Returns the command to compile a vhdl file
         """
@@ -188,8 +193,8 @@ class ModelSimInterface(SimulatorInterface):  # pylint: disable=too-many-instanc
             coverage_args = ["+cover=" + to_coverage_args(self._coverage)]
 
         return ([join(self._prefix, 'vcom'), '-quiet', '-modelsimini', self._modelsim_ini] +
-                coverage_args + compile_options.get("modelsim.vcom_flags", []) +
-                ['-' + self._vhdl_standard, '-work', library_name, source_file_name])
+                coverage_args + source_file.compile_options.get("modelsim.vcom_flags", []) +
+                ['-' + source_file.get_vhdl_standard(), '-work', source_file.library.name, source_file.name])
 
     def compile_verilog_file_command(self, source_file):
         """
@@ -268,7 +273,11 @@ class ModelSimInterface(SimulatorInterface):  # pylint: disable=too-many-instanc
         else:
             coverage_file = join(output_path, "coverage.ucdb")
             self._coverage_files.add(coverage_file)
-            coverage_save_cmd = "coverage save -onexit -assert -directive -cvg -codeAll {%s}" % fix_path(coverage_file)
+            coverage_save_cmd = "coverage save -onexit -testname {%s} -assert -directive -cvg -codeAll {%s}" % (
+                # Ugly output path to test name translation until this is passed to simulator interfaces
+                HASH_TO_TEST_NAME[os.path.basename(os.path.dirname(output_path))],
+                fix_path(coverage_file))
+
             coverage_args = "-coverage=" + to_coverage_args(self._coverage)
 
         vsim_flags = ["-wlf {%s}" % fix_path(join(output_path, "vsim.wlf")),
@@ -396,6 +405,20 @@ proc vunit_run {} {
 }
 """ % (1 if fail_on_warning else 2, no_warnings, no_warnings)
 
+    @staticmethod
+    def _create_restart_function():
+        """"
+        Create the vunit_restart function to recompile and restart the simulation
+        """
+        tcl = """
+proc vunit_restart {} {
+    echo [exec -ignorestderr %s %s --compile]
+    restart -f
+    vunit_run
+}
+""" % (fix_path(sys.executable), fix_path(sys.argv[0]))
+        return tcl
+
     def _vsim_extra_args(self, config):
         """
         Determine vsim_extra_args
@@ -427,12 +450,16 @@ proc vunit_help {} {
     echo {  - Optional first argument are passed as extra flags to vsim}
     echo {vunit_run}
     echo {  - Run test, must do vunit_load first}
+    echo {vunit_restart}
+    echo {  - Recompiles the source files}
+    echo {  - Restarts the simulation and does vunit_run}
 }
 """
         tcl += self._create_load_function(library_name, entity_name, architecture_name,
                                           config, output_path)
         tcl += self._create_run_function(config.fail_on_warning,
                                          config.disable_ieee_warnings)
+        tcl += self._create_restart_function()
         return tcl
 
     @staticmethod
@@ -609,11 +636,11 @@ if {![vunit_load -vhdlvariablelogging]} {
         self.teardown()
 
         merged_coverage_file = join(output_path, "merged_coverage.ucdb")
-        vcover_cmd = "vcover merge -strip 1 " + merged_coverage_file
+        vcover_cmd = [join(self._prefix, 'vcover'), 'merge', '-strip', '1', merged_coverage_file]
 
         for coverage_file in self._coverage_files:
             if file_exists(coverage_file):
-                vcover_cmd = vcover_cmd + " " + coverage_file
+                vcover_cmd.append(coverage_file)
             else:
                 LOGGER.warning("Missing coverage ucdb file: %s", coverage_file)
 
